package com.sunlabs.archsim.classRegistry;

import java.util.Collection;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Map.Entry;

public class ComponentInfo {

	
	protected Class componentType;
	protected String componentTypeName;
	protected LinkedList<String> tags;
	protected LinkedList<String> inputs;
	protected LinkedList<String> outputs;
	protected HashMap<String, String> parameters;
	protected HashMap<String, String> globals;
	
	public ComponentInfo() {
		componentType= null;
		componentTypeName= null;
		tags= new LinkedList<String>();
		inputs= new LinkedList<String>();
		outputs= new LinkedList<String>();		
		parameters= new HashMap<String, String>();
		globals= new HashMap<String, String>();
	}
	
	void setClass(Class c) {
		componentType= c;
	}
	
	void setName(String n) {
		componentTypeName= n;
	}
	
	public void addTag(String tag) {
		tags.add(tag);
	}
	
	public void addInput(String input) {
		inputs.add(input);
	}

	public void addOutput(String input) {
		outputs.add(input);
	}
	
	public void addParameter(String name, String type) {
		parameters.put(name, type);
	}
	
	public void addGlobal(String name, String type) {
		globals.put(name, type);
	}
	
	public Class getComponentType() {
		return componentType;
	}
	
	public String getName() {
		return componentTypeName;
	}

	public Collection getTags() {
		return tags;
	}
	
	public Collection getInputs() {
		return inputs;
	}
	
	public Collection getOutputs() {
		return outputs;
	}
	
	public Collection getParameters() {
		return parameters.keySet();
	}
	
	public String getParameterType(String parameter) {
		return parameters.get(parameter);
	}
	
	public Collection getGlobals() {
		return globals.keySet();
	}
	
	public String getGlobalType(String global) {
		return globals.get(global);
	}
	
	public String toString() {
		String result= "Component info: \n";
		result+= "\tName: " + componentTypeName + "\n";
		result+= "\tClass: " + componentType.getName() + "\n";
		
		String tagString="\tTags: [ ";
		for (String tag: tags) {
			tagString+= " "+ tag + " ";
		}
		tagString+= " ]";
		result+= tagString + "\n";

		String inputString="\tInputs: [ ";
		for (String input: inputs) {
			inputString+= " "+ input + " ";
		}
		inputString+= " ]";
		result+= inputString + "\n";

		String outputString="\tOutputs: [ ";
		for (String output: outputs) {
			outputString+= " "+ output + " ";
		}
		outputString+= " ]";
		result+= outputString + "\n";

		String parameterString= "\tParameters: [ ";
		for (Entry e : parameters.entrySet()) {
			parameterString+= " ( " + e.getKey() + ", " + e.getValue() + " )";
		}
		parameterString+= " ]";
		result+= parameterString + "\n";
		
		String globalString= "\tGlobals: [ ";
		for (Entry e : globals.entrySet()) {
			globalString+= " ( " + e.getKey() + ", " + e.getValue() + " )";
		}
		globalString+= " ]";
		result+= globalString;
		
		return result;
	}
	
} // class ComponentInfo
